<?php

namespace Drupal\mapbox_field\Plugin\Field\FieldType;

use Drupal\Component\Utility\Random;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Plugin implementation of the 'mapbox_field' field type.
 *
 * @FieldType(
 *   id = "mapbox_field",
 *   label = @Translation("Mapbox Field"),
 *   description = @Translation("Mapbox Field"),
 *   default_widget = "mapbox_widget",
 *   default_formatter = "mapbox_formatter"
 * )
 */
class MapboxField extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['lat'] = DataDefinition::create('string')
    ->setLabel(new TranslatableMarkup('Latitude'))
    ->setRequired(FALSE);
    $properties['lng'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Longitude'))
      ->setRequired(FALSE);
    $properties['marker'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Marker'))
      ->setRequired(FALSE);
    $properties['markerText'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Marker Text'))
      ->setRequired(FALSE);
    $properties['marked'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Marked'))
      ->setRequired(FALSE);
    $properties['zoom'] = DataDefinition::create('string')
    ->setLabel(new TranslatableMarkup('Zoom'))
    ->setRequired(FALSE);
    $properties['threeDBuildings'] = DataDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('3D Buildings'))
      ->setRequired(FALSE);
    $properties['keyboard'] = DataDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('Keyboard'))
      ->setRequired(FALSE);
    $properties['minZoom'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Min Zoom'))
      ->setRequired(FALSE);
    $properties['maxZoom'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Max Zoom'))
      ->setRequired(FALSE);
    $properties['maxPitch'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Max Pitch'))
      ->setRequired(FALSE);
    $properties['pitch'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Pitch'))
      ->setRequired(FALSE);
    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    $schema = [
      'columns' => [
        'lat' => [
          'type' => 'float',
          'size' => 'big',
          'default' => 0.0,
          'not null' => FALSE,
        ],
        'lng' => [
          'type' => 'float',
          'size' => 'big',
          'default' => 0.0,
          'not null' => FALSE,
        ],
        'marker' => [
          'type' => 'varchar',
          'length' => 255,
          'default' => '',
          'not null' => FALSE,
        ],
        'markerText' => [
          'type' => 'varchar',
          'length' => 255,
          'default' => '',
          'not null' => FALSE,
        ],
        'marked' => [
          'type' => 'int',
          'length' => 10,
          'not null' => FALSE,
        ],
        'zoom' => [
          'type' => 'int',
          'length' => 10,
          'not null' => FALSE,
        ],
        'threeDBuildings' => [
          'type' => 'int',
          'length' => 10,
          'not null' => FALSE,
        ],
        'keyboard' => [
          'type' => 'int',
          'length' => 10,
          'not null' => FALSE,
        ],
        'minZoom' => [
          'type' => 'int',
          'length' => 10,
          'not null' => FALSE,
        ],
        'maxZoom' => [
          'type' => 'int',
          'length' => 10,
          'not null' => FALSE,
        ],
        'maxPitch' => [
          'type' => 'int',
          'length' => 10,
          'not null' => FALSE,
        ],
        'pitch' => [
          'type' => 'int',
          'length' => 10,
          'not null' => FALSE,
        ],
      ],
    ];

    return $schema;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $value = $this->get('lat')->getValue();
    return $value === NULL || $value === '';
  }

}
